@@ -23,6 +23,7 @@ from rich.table import Table
 
 from dotbot import GATEWAY_ADDRESS_DEFAULT
 from dotbot.hdlc import HDLCHandler, HDLCState, hdlc_encode
+from dotbot.logger import LOGGER
 from dotbot.protocol import (
     ProtocolPayload,
     ProtocolHeader,
@@ -47,6 +48,10 @@ from dotbot.models import (
     DotBotStatus,
     DotBotLH2Position,
     DotBotGPSPosition,
+    DotBotNotificationCommand,
+    DotBotNotificationModel,
+    DotBotNotificationUpdate,
+    MAX_POSITION_HISTORY_SIZE,
 )
 from dotbot.server import web
 from dotbot.lighthouse2 import LighthouseManager, LighthouseManagerState
@@ -55,9 +60,8 @@ from dotbot.lighthouse2 import LighthouseManager, LighthouseManagerState
 CONTROLLERS = {}
 LOST_DELAY = 5  # seconds
 DEAD_DELAY = 60  # seconds
-MAX_POSITION_HISTORY_SIZE = 1000
 LH2_POSITION_DISTANCE_THRESHOLD = 0.01
-GPS_POSITION_DISTANCE_THRESHOLD = 10  # meters
+GPS_POSITION_DISTANCE_THRESHOLD = 5  # meters
 
 
 class ControllerException(Exception):
@@ -127,10 +131,11 @@ class ControllerBase(ABC):
             version=PROTOCOL_VERSION,
         )
         self.settings = settings
-        self.hdlc_handler = HDLCHandler(verbose=settings.verbose)
+        self.hdlc_handler = HDLCHandler()
         self.serial = None
         self.websockets = []
         self.lh2_manager = LighthouseManager()
+        self.logger = LOGGER.bind(context=__name__)
 
     @abstractmethod
     def init(self):
@@ -168,7 +173,9 @@ class ControllerBase(ABC):
                 writer.close()
                 break
         if self.settings.webbrowser is True:
-            webbrowser.open("http://localhost:8000/dotbots")
+            url = "http://localhost:8000/dotbots"
+            self.logger.info("Opening webbrowser", url=url)
+            webbrowser.open(url)
 
     async def _dotbots_status_refresh(self):
         """Coroutine that periodically updates the status of known dotbot."""
@@ -184,7 +191,9 @@ class ControllerBase(ABC):
                     dotbot.status = DotBotStatus.ALIVE
                 needs_refresh = previous_status != dotbot.status
             if needs_refresh is True:
-                await self.notify_clients(json.dumps({"cmd": "reload"}))
+                await self.notify_clients(
+                    DotBotNotificationModel(cmd=DotBotNotificationCommand.RELOAD)
+                )
             await asyncio.sleep(1)
 
     async def _dotbots_table_refresh(self):
@@ -237,14 +246,15 @@ class ControllerBase(ABC):
                 try:
                     payload = ProtocolPayload.from_bytes(payload)
                 except ProtocolPayloadParserException:
+                    self.logger.warning("Cannot parse payload")
                     if self.settings.verbose is True:
-                        print(f"Cannot parse payload '{payload}'")
+                        print(payload)
                     return
                 self.handle_received_payload(payload)
 
     def handle_received_payload(
         self, payload: ProtocolPayload
-    ):  # pylint:disable=too-many-branches
+    ):  # pylint:disable=too-many-branches,too-many-statements
         """Handle a received payload."""
         # Controller is not interested by command messages received
         if payload.payload_type in [
@@ -252,11 +262,14 @@ class ControllerBase(ABC):
             PayloadType.CMD_RGB_LED,
         ]:
             return
-        if self.settings.verbose is True:
-            print(payload)
         source = hexlify(int(payload.header.source).to_bytes(8, "big")).decode()
+        logger = self.logger.bind(
+            source=source,
+            payload_type=payload.payload_type.name,
+            application=payload.header.application.name,
+        )
         if source == GATEWAY_ADDRESS_DEFAULT:
-            print(f"Invalid source in payload type {payload.payload_type}")
+            logger.warning("Invalid source in payload")
             return
         dotbot = DotBotModel(
             address=source,
@@ -264,7 +277,7 @@ class ControllerBase(ABC):
             last_seen=time.time(),
             active=(int(source, 16) == self.header.destination),
         )
-        should_reload = False
+        notification_cmd = DotBotNotificationCommand.NONE
         if source in self.dotbots:
             dotbot.mode = self.dotbots[source].mode
             dotbot.direction = self.dotbots[source].direction
@@ -274,10 +287,16 @@ class ControllerBase(ABC):
             dotbot.waypoints = self.dotbots[source].waypoints
             dotbot.waypoints_threshold = self.dotbots[source].waypoints_threshold
             dotbot.position_history = self.dotbots[source].position_history
-            should_reload = dotbot.status != self.dotbots[source].status
         else:
-            # only reload if a new dotbot comes in
-            should_reload = True
+            # reload if a new dotbot comes in
+            notification_cmd = DotBotNotificationCommand.RELOAD
+
+        if (
+            payload.payload_type in [PayloadType.DOTBOT_DATA, PayloadType.SAILBOT_DATA]
+            and -500 <= payload.values.direction <= 500
+        ):
+            dotbot.direction = payload.values.direction
+            logger = logger.bind(direction=dotbot.direction)
 
         dotbot.lh2_position = self._compute_lh2_position(payload)
         if (
@@ -290,26 +309,16 @@ class ControllerBase(ABC):
                 y=dotbot.lh2_position.y,
                 z=dotbot.lh2_position.z,
             )
+            logger.info("lh2", x=dotbot.lh2_position.x, y=dotbot.lh2_position.y)
             if (
                 not dotbot.position_history
                 or lh2_distance(dotbot.position_history[-1], new_position)
                 >= LH2_POSITION_DISTANCE_THRESHOLD
             ):
                 dotbot.position_history.append(new_position)
+                notification_cmd = DotBotNotificationCommand.UPDATE
             if len(dotbot.position_history) > MAX_POSITION_HISTORY_SIZE:
                 dotbot.position_history.pop(0)
-            asyncio.create_task(
-                self.notify_clients(
-                    json.dumps(
-                        {
-                            "cmd": "lh2_position",
-                            "address": dotbot.address,
-                            "x": dotbot.lh2_position.x,
-                            "y": dotbot.lh2_position.y,
-                        }
-                    )
-                )
-            )
             # Send the computed position back to the dotbot
             header = ProtocolHeader(
                 destination=int(source, 16),
@@ -330,29 +339,13 @@ class ControllerBase(ABC):
                 )
             )
 
-        if (
-            payload.payload_type in [PayloadType.DOTBOT_DATA, PayloadType.SAILBOT_DATA]
-            and -500 <= payload.values.direction <= 500
-        ):
-            dotbot.direction = payload.values.direction
-            asyncio.create_task(
-                self.notify_clients(
-                    json.dumps(
-                        {
-                            "cmd": "direction",
-                            "address": dotbot.address,
-                            "direction": dotbot.direction,
-                        }
-                    )
-                )
-            )
-
         if payload.payload_type in [PayloadType.GPS_POSITION, PayloadType.SAILBOT_DATA]:
             new_position = DotBotGPSPosition(
                 latitude=float(payload.values.latitude) / 1e6,
                 longitude=float(payload.values.longitude) / 1e6,
             )
             dotbot.gps_position = new_position
+            logger.info("gps", lat=new_position.latitude, long=new_position.longitude)
             if (
                 not dotbot.position_history
                 or gps_distance(dotbot.position_history[-1], new_position)
@@ -361,22 +354,26 @@ class ControllerBase(ABC):
                 dotbot.position_history.append(new_position)
             if len(dotbot.position_history) > MAX_POSITION_HISTORY_SIZE:
                 dotbot.position_history.pop(0)
-            asyncio.create_task(
-                self.notify_clients(
-                    json.dumps(
-                        {
-                            "cmd": "gps_position",
-                            "address": dotbot.address,
-                            "latitude": dotbot.gps_position.latitude,
-                            "longitude": dotbot.gps_position.longitude,
-                        }
-                    )
-                )
-            )
+            notification_cmd = DotBotNotificationCommand.UPDATE
 
+        if notification_cmd == DotBotNotificationCommand.UPDATE:
+            notification = DotBotNotificationModel(
+                cmd=notification_cmd.value,
+                data=DotBotNotificationUpdate(
+                    address=dotbot.address,
+                    direction=dotbot.direction,
+                    lh2_position=dotbot.lh2_position,
+                    gps_position=dotbot.gps_position,
+                ),
+            )
+        else:
+            notification = DotBotNotificationModel(cmd=notification_cmd.value)
+
+        if self.settings.verbose is True:
+            print(payload)
         self.dotbots.update({dotbot.address: dotbot})
-        if should_reload is True:
-            asyncio.create_task(self.notify_clients(json.dumps({"cmd": "reload"})))
+        if notification_cmd != DotBotNotificationCommand.NONE:
+            asyncio.create_task(self.notify_clients(notification))
 
     async def _ws_send_safe(self, websocket: WebSocket, msg: str):
         """Safely send a message to a websocket client."""
@@ -385,10 +382,16 @@ class ControllerBase(ABC):
         except websockets.exceptions.ConnectionClosedError:
             await asyncio.sleep(0.1)
 
-    async def notify_clients(self, message):
+    async def notify_clients(self, notification):
         """Send a message to all clients connected."""
+        self.logger.debug("notify", cmd=notification.cmd.name)
         await asyncio.gather(
-            *[self._ws_send_safe(websocket, message) for websocket in self.websockets]
+            *[
+                self._ws_send_safe(
+                    websocket, json.dumps(notification.dict(exclude_none=True))
+                )
+                for websocket in self.websockets
+            ]
         )
 
     def send_payload(self, payload: ProtocolPayload):
@@ -402,6 +405,12 @@ class ControllerBase(ABC):
         payload.header.application = self.dotbots[destination].application
         if self.serial is not None:
             self.serial.write(hdlc_encode(payload.to_bytes()))
+            self.logger.debug(
+                "Payload sent",
+                application=payload.header.application.name,
+                destination=destination,
+                payload_type=payload.payload_type.name,
+            )
 
     def get_dotbots(self, query: DotBotQueryModel) -> List[DotBotModel]:
         """Returns the list of dotbots matching the query."""
@@ -440,7 +449,7 @@ class ControllerBase(ABC):
             SerialInterfaceException,
             serial.serialutil.SerialException,
         ) as exc:
-            print(f"{exc}")
+            self.logger.info(f"Stopping controller {exc}")
             for task in tasks:
                 task.cancel()
 
